//+------------------------------------------------------------------+
//|                                                      OLAPGUI.mq5 |
//|                               Copyright (c) 2019-2020, Marketeer |
//|                          https://www.mql5.com/en/users/marketeer |
//|               Online Analytical Processing of trading hypercubes |
//|                            https://www.mql5.com/ru/articles/6602 |
//|                            https://www.mql5.com/ru/articles/6603 |
//|                            https://www.mql5.com/ru/articles/7656 |
//+------------------------------------------------------------------+
#property copyright "Copyright (c) 2019-2020, Marketeer"
#property link "https://www.mql5.com/en/users/marketeer"
#property version "1.0"


#include <OLAP/OLAPTradesCustom.mqh>
#include <OLAP/HTMLcube.mqh>
#include <OLAP/CSVcube.mqh>
#include <OLAP/TSTcube.mqh>
#include <OLAP/GUI/OLAPGUI_Trades.mqh>

input GroupSettings Log_Settings; // L O G G I N G
input bool PrintDetailedLogs = false; // · PrintDetailedLogs

OLAPDialog<SELECTORS,ENUM_FIELDS> dialog(_defaultEngine);


int OnInit()
{
  _defaultEngine.setShortTitles(true);
  dialog.setPrintLog(PrintDetailedLogs);
  if(ReportFile == "")
  {
    Print("Analyzing account history");
    _defaultEngine.setAdapter(&_defaultHistoryAdapter);
  }
  else
  {
    if(StringFind(ReportFile, ".htm") > 0 && _defaultHTMLReportAdapter.load(ReportFile))
    {
      _defaultEngine.setAdapter(&_defaultHTMLReportAdapter);
    }
    else
    if(StringFind(ReportFile, ".csv") > 0 && _defaultCSVReportAdapter.load(ReportFile))
    {
      _defaultEngine.setAdapter(&_defaultCSVReportAdapter);
    }
    else
    if(StringFind(ReportFile, ".tst") > 0 && _defaultTSTReportAdapter.load(ReportFile))
    {
      _defaultEngine.setAdapter(&_defaultTSTReportAdapter);
    }
    else
    {
      Print("Unknown file format: ", ReportFile);
      return INIT_PARAMETERS_INCORRECT;
    }
  }

  Print("\n");
  int w = (int)ChartGetInteger(0, CHART_WIDTH_IN_PIXELS);
  int h = (int)ChartGetInteger(0, CHART_HEIGHT_IN_PIXELS);
  bool screenshot = GlobalVariableCheck("OLAPGUISCREENSHOT");
  if(!screenshot)
  {
    if(!dialog.Create(0, "OLAPGUI" + (ReportFile != "" ? " : " + ReportFile : ""), 0,  w / 6, h / 6, w / 6 + w * 2 / 3, h / 6 + h * 2 / 3)) return INIT_FAILED;
  }
  else
  {
    if(!dialog.Create(0, "OLAPGUI" + (ReportFile != "" ? " : " + ReportFile : ""), 0,  0, 0, 750, 492)) return INIT_FAILED;
  }
  if(!dialog.Run()) return INIT_FAILED;
  return INIT_SUCCEEDED;
}

void OnDeinit(const int reason)
{
  dialog.Destroy(reason);
}

void OnChartEvent(const int id,
                  const long &lparam,
                  const double &dparam,
                  const string &sparam)
{
  dialog.ChartEvent(id, lparam, dparam, sparam);
}
//+------------------------------------------------------------------+
