//+------------------------------------------------------------------+
//|                                                OLAP_OPTS_GUI.mq5 |
//|                                 Copyright © 2016-2020, Marketeer |
//|                          https://www.mql5.com/en/users/marketeer |
//|               Online Analytical Processing of trading hypercubes |
//|                            https://www.mql5.com/ru/articles/6602 |
//|                            https://www.mql5.com/ru/articles/6603 |
//|                            https://www.mql5.com/ru/articles/7535 |
//|                            https://www.mql5.com/ru/articles/7656 |
//+------------------------------------------------------------------+

#property copyright "Copyright © 2016-2020, Marketeer"
#property link      "https://www.mql5.com/ru/articles/7535"
#property version   "1.0"
#property description "Optimization Cache Records Analysis: Non-trading Expert Adviser demonstrating OLAP classes"


input string OptFileName = "Integrity.opt";
input bool PrintDetailedLogs = false;
input uint QuantGranularity = 100;


#include <OLAP/OLAPOpts.mqh>
#include <OLAP/GUI/OLAPGUI_Opts.mqh>


OLAPDialog<SELECTORS,ENUM_FIELDS> dialog(_defaultEngine);


int OnInit()
{
  _defaultEngine.setAdapter(&_defaultOptCacheAdapter);
  _defaultEngine.setShortTitles(true);
  _defaultEngine.setQuant(QuantGranularity);
  _defaultOptCacheAdapter.load(OptFileName);
  dialog.setCustomFields(_defaultOptCacheAdapter);
  dialog.setPrintLog(PrintDetailedLogs);

  const int w = (int)ChartGetInteger(0, CHART_WIDTH_IN_PIXELS);
  const int h = (int)ChartGetInteger(0, CHART_HEIGHT_IN_PIXELS);
  const bool screenshot = GlobalVariableCheck("OLAPGUISCREENSHOT");
  if(!screenshot)
  {
    if(!dialog.Create(0, "OLAPGUI" + (OptFileName != "" ? " : " + OptFileName : ""), 0,  w / 6, h / 6, w / 6 + w * 2 / 3, h / 6 + h * 2 / 3)) return INIT_FAILED;
  }
  else
  {
    if(!dialog.Create(0, "OLAPGUI" + (OptFileName != "" ? " : " + OptFileName : ""), 0,  0, 0, 750, 560)) return INIT_FAILED;
  }
  if(!dialog.Run()) return INIT_FAILED;
  
  return INIT_SUCCEEDED;
}

void OnDeinit(const int reason)
{
  dialog.Destroy(reason);
}

void OnChartEvent(const int id,
                  const long &lparam,
                  const double &dparam,
                  const string &sparam)
{
  dialog.ChartEvent(id, lparam, dparam, sparam);
}

void OnTick()
{
}
