#include "String.mqh"

#define UINT64 ulong
#define INT64 datetime
#define UINT uint

//+------------------------------------------------------------------+
//|                                           |
//+------------------------------------------------------------------+
struct TradeDeal
{
  UINT64            deal;                    //   
  UINT64            order;                   //     
  INT64             time_create;             //   
  //   wchar_t           symbol[32];              //     
  STRING32          symbol;                  //     
  double            price_open;              //  \
  double            price_close;             //  
  double            sl;                      // - 
  double            tp;                      // - 
  UINT64            volume;                  //  
  double            profit;                  //  
  double            commission;              //   
  double            reserve;                 // 
  double            storage;                 // 
  //   wchar_t           comment[32];             //   
  STRING32          comment;                 //   
  ENUM_DEAL_TYPE    action;                  //  
  ENUM_DEAL_ENTRY   entry;                   //     
  UINT              digits;                  //     
  UINT              digits_currency;         //     
  double            contract_size;           //  

  bool Set( const ulong Ticket )
  {
    const bool Res = (::HistoryDealGetInteger(Ticket, DEAL_TICKET) == Ticket);

    if (Res)
    {
      this.deal = Ticket;                                                                                    //   

      string Str = ::HistoryDealGetString(Ticket, DEAL_SYMBOL);;
      this.symbol = Str;                                                                                     //     

      this.contract_size = ::SymbolInfoDouble(Str, SYMBOL_TRADE_CONTRACT_SIZE);                              //  
      this.digits = (UINT)::SymbolInfoInteger(Str, SYMBOL_DIGITS);                                           //     

      Str = ::HistoryDealGetString(Ticket, DEAL_COMMENT);
      this.comment = Str;                                                                                    //   

      this.order = ::HistoryDealGetInteger(Ticket, DEAL_ORDER);                                              //     
      this.time_create = (INT64)::HistoryDealGetInteger(Ticket, DEAL_TIME);                                  //   
      this.price_open = ::HistoryDealGetDouble(Ticket, DEAL_PRICE);                                          //  \
      this.price_close = 0;                                                                                  //  
      this.sl = 0;                                                                                           // - 
      this.tp = 0;                                                                                           // - 
      this.volume = (UINT64)(::HistoryDealGetDouble(Ticket, DEAL_VOLUME) * this.contract_size * 1000 + 0.1); //  
      this.profit = ::HistoryDealGetDouble(Ticket, DEAL_PROFIT);                                             //  
      this.commission = ::HistoryDealGetDouble(Ticket, DEAL_COMMISSION);                                     //   
      this.reserve = 0;                                                                                      // 
      this.storage = ::HistoryDealGetDouble(Ticket, DEAL_SWAP);                                              // 
      this.action = (ENUM_DEAL_TYPE)::HistoryDealGetInteger(Ticket, DEAL_TYPE);                              //  
      this.entry = (ENUM_DEAL_ENTRY)::HistoryDealGetInteger(Ticket, DEAL_ENTRY);                             //     
      this.digits_currency = (UINT)::AccountInfoInteger(ACCOUNT_CURRENCY_DIGITS);                            //     
    }

    return(Res);
  }

  long GetProperty( const ENUM_DEAL_PROPERTY_INTEGER Property ) const
  {
    long Res = 0;

    switch (Property)
    {
      case DEAL_TICKET:
        Res = (long)this.deal;

        break;
      case DEAL_ORDER:
        Res = (long)this.order;

        break;
      case DEAL_TIME:
        Res = this.time_create;

        break;
      case DEAL_TIME_MSC:
        Res = (long)this.time_create * 1000;

        break;
      case DEAL_TYPE:
        Res = this.action;

        break;
      case DEAL_ENTRY:
        Res = this.entry;

        break;
      case DEAL_POSITION_ID:
        Res = (long)this.order;

        break;
    }

    return(Res);
  }

  double GetProperty( const ENUM_DEAL_PROPERTY_DOUBLE Property ) const
  {
    double Res = 0;

    switch (Property)
    {
      case DEAL_VOLUME:
        Res = (double)this.volume / (this.contract_size ? this.contract_size * 1000 : 1e8);

        break;
      case DEAL_PRICE:
        Res = this.price_open;

        break;
      case DEAL_COMMISSION:
        Res = this.commission;

        break;
      case DEAL_SWAP:
        Res = this.storage;

        break;
      case DEAL_PROFIT:
        Res = this.profit;

        break;
    }

    return(Res);
  }

  string GetProperty( const ENUM_DEAL_PROPERTY_STRING Property ) const
  {
    string Res = NULL;

    switch (Property)
    {
      case DEAL_SYMBOL:
        Res = this.symbol[];

        break;
      case DEAL_COMMENT:
        Res = this.comment[];

        break;
    }

    return(Res);
  }

#define TOSTRING(A) #A + " = " + (string)(this.A) + "\n"
#define TOSTRING2(A) #A + " = " + ::EnumToString(this.A) + " (" + (string)(this.A) + ")\n"
#define TOSTRING3(A) #A + " = " + this.A[] + "\n"

  string ToString( void ) const
  {
    return(
           TOSTRING(deal) +                    //   
           TOSTRING(order) +                   //     
           TOSTRING(time_create) +             //   
           TOSTRING3(symbol) +                 //     
           TOSTRING(price_open) +              //  \
           TOSTRING(price_close) +             //  
           TOSTRING(sl) +                      // - 
           TOSTRING(tp) +                      // - 
           TOSTRING(volume) +                  //  
           TOSTRING(profit) +                  //  
           TOSTRING(commission) +              //   
           TOSTRING(reserve) +                 // 
           TOSTRING(storage) +                 // 
           TOSTRING3(comment) +                //   
           TOSTRING2(action) +                 //  
           TOSTRING2(entry) +                  //     
           TOSTRING(digits) +                  //     
           TOSTRING(digits_currency) +         //     
           TOSTRING(contract_size)             //  
          );
  }

#undef TOSTRING3
#undef TOSTRING2
#undef TOSTRING
};

#undef UINT
#undef INT64
#undef UINT64